#! /usr/local/bin/python
"""Get version and location of relevant mmCIF dictionaries for ModelCIF.

Fetch the versions of the ModelCIF dictionary and the PDBx/mmCIF dictionary used
to build it into a JSON file.
"""
# pylint: disable=invalid-name
# pylint: enable=invalid-name

import argparse
import sys

import rapidjson as json

from mmcif.io.IoAdapterPy import IoAdapterPy


def _parse_command_line():
    """Get arguments."""
    parser = argparse.ArgumentParser(description=__doc__)

    parser.add_argument(
        "dic_file",
        type=str,
        metavar="<DICTIONARY FILE>",
        help="The mmCIF dictionary file to read the versions from.",
    )
    parser.add_argument(
        "--parent",
        "-p",
        type=str,
        metavar="<NAME OF PARENT DICT>",
        help="Name of to the 'parent' dictionary. This is the one the other "
        + "dictionary is appended to. This is usually the mmcif_pdbx_v50.dic.",
        default="mmcif_pdbx_v50.dic",
    )
    parser.add_argument(
        "--output",
        "-o",
        type=str,
        metavar="<PATH TO VERSION FILE>",
        help="Path to store the JSON file with the version at.",
        default="mmcif_ma_version.json",
    )
    parser.add_argument(
        "--parent-location",
        "-u",
        type=str,
        metavar="<URL OF PARENT DICT FILE>",
        help="Download location of the parent dictionary file.",
        default=None,
    )
    parser.add_argument(
        "--child-location",
        "-l",
        type=str,
        metavar="<URL OF CHILD DICT FILE>",
        help="Download location of the child dictionary file.",
        default=None,
    )
    opts = parser.parse_args()

    return opts


def _error(msg):
    """Print a final error message."""
    print(msg + "\nAborting.", file=sys.stderr)
    sys.exit(1)


def _get_data_cat(cat, file_name, data):
    """Get a data category from a mmCIF data blob."""
    obj = data.getObj(cat)
    if obj is None:
        _error(f"No '{cat}' object found in '{file_name}'.")

    return obj


def _get_data_item(itm, cat, file_name, cat_data):
    """Get a single data item from a data category."""
    val = cat_data.getAttributeValueList(itm)
    if len(val) != 1:
        _error(
            f"Expected exactly 1 '{cat}.{itm}' in '{file_name}', "
            + f"found '{', '.join(val)}'."
        )

    return val[0]


def _get_versions(dic_file, parent_name, io_adapter):
    """Fetch the 'category_group_list' object and assemble a version for the
    dictionary."""

    dic = io_adapter.readFile(inputFilePath=dic_file)

    # fetch a data container from the list returned by the adapter
    cntnr = None
    for obj in dic:
        if "dictionary" in obj.getObjNameList():
            cntnr = obj
            break

    if cntnr is None:
        _error(f"No 'dictionary' object found in '{dic_file}'.")

    dic = _get_data_cat("dictionary", dic_file, cntnr)

    vrsn = _get_data_item("version", "dictionary", dic_file, dic)
    ttl = _get_data_item("title", "dictionary", dic_file, dic)
    dic_version = {"title": ttl, "version": vrsn}

    cmp = _get_data_cat("pdbx_dictionary_component", dic_file, cntnr)
    dc_idx = cmp.getAttributeIndex("dictionary_component_id")
    vs_idx = cmp.getAttributeIndex("version")
    for row in cmp:
        if row[dc_idx] == parent_name:
            vrsn = row[vs_idx]
            prnt_version = {"title": parent_name, "version": vrsn}
            break

    return dic_version, prnt_version


def _add_dict_location(parent, child, parent_loc, child_loc):
    """Add URLs to the dictionary versions if available."""
    if parent_loc is None:
        parent["location"] = "."
    else:
        parent["location"] = parent_loc
    if child_loc is None:
        child["location"] = "."
    else:
        child["location"] = child_loc


def _main():
    """Run as script."""
    opts = _parse_command_line()

    io_adapter = IoAdapterPy(False, sys.stdout)
    c_vrsn, p_vrsn = _get_versions(opts.dic_file, opts.parent, io_adapter)

    _add_dict_location(
        p_vrsn, c_vrsn, opts.parent_location, opts.child_location
    )
    with open(opts.output, "w", encoding="utf8") as jfh:
        json.dump({"versions": [p_vrsn, c_vrsn]}, jfh)


if __name__ == "__main__":
    _main()

#  LocalWords:  DictToSdb SDB PDBx CifCheck pylint mmcif pdbx dic nAborting
#  LocalWords:  macromolecular utf
